local Class = require 'class'
local calc = true
local regions = {}
local M = Class()

function M:init(map_width, map_height, block_size, seed, pointer)
	self.loaded = false
	self.map_width = map_width
	self.map_height = map_height
	self.seed = seed
	self.map = {}
	self.block_size = block_size
	self.scale = 10
	self.threshold = 40
	for x=1,self.map_width do
		self.map[x] = {}
		for y=1,self.map_height do
			self.map[x][y] = {}
			self.map[x][y].type = 0
			self.map[x][y].looked = false
		end
	end
	love.math.setRandomSeed(self.seed)
	self:rand()
	self:smooth(5)


	self:collect_regions(1,1,_)

	for i,v in pairs(regions) do
		local t = regions[i].type
		for d,v in ipairs(regions[i]) do
			self.map[v.x][v.y].type = t
		end
	end

	print(#regions)
	self:set_cunks()
	self.map = nil
	self.pointer = pointer
	self.loaded = true
end
function M:grid_get(x,y) return self.map[x][y].type end
function M:set(x,y,v) self.map[x][y].type = v end
function M:collect_regions(x,y,v)
	local region = {}
	region.type = self:grid_get(x,y)
    if self:grid_get(x,y) == v then return end
    local oldv = self:grid_get(x,y)
    local qx = {x}
    local qy = {y}
    local left = 0
    local right = 1
    
    local function enqueue(x,y)
        right = right + 1
        qx[right], qy[right] = x, y
    end
    
    while left < right do
        left = left + 1
        x, y = qx[left], qy[left]
        if x >= 1 and y >= 1 and x <= self.map_width and y <= self.map_width then
        	if self:grid_get(x, y) == oldv then
        		local r = {x=x,y=y}
        		table.insert(region,r)
        		self:set(x, y, v)
        		self.map[x][y].looked = true
        		enqueue(x+1, y)
        		enqueue(x-1, y)
        		enqueue(x, y+1)
        		enqueue(x, y-1)
        	end
        end
    end
    table.insert(regions, region)
    local t = self:get_next()
    if t ~= nil then
    	-- print(t.x,t.y)
    	self:collect_regions(t.x,t.y,_)
	end

end

function M:get_next()
	local x = self.prevx or 1
	local y = self.prevy or 1
	local max_x = self.map_width
	local max_y = self.map_width

	repeat 
		y = y+1
		if y > max_y then
			y = 1
			x = x+1
		end
		if x > max_x then
			return
		end
	until self.map[x][y].looked == false
	self.prevx, self.prevy = x,y
	local t = {x=x,y=y}
	return t
end



function M:set_active(ix,iy)
	for i=1,self.number_of_chunks_x do
		for o=1,self.number_of_chunks_y do
			if i == ix and o == iy or i == ix-1 and o == iy-1 or i == ix and o == iy-1 or i == ix+1 and o == iy-1 or i == ix-1 and o == iy or i == ix+1 and o == iy or i == ix-1 and o == iy+1 or i == ix and o == iy+1 or i == ix+1 and o == iy+1 then
				self.chunks[i][o].active = true

			else
				self.chunks[i][o].active = false
			end
		end
	end
end


function M:set_cunks()
	self.current_chunk = 1
	self.chunk_size = 16
	local cx = self.map_width/self.chunk_size
	local cy = self.map_height/self.chunk_size
	self.number_of_chunks_x = cx
	self.number_of_chunks_y = cy
	self.chunks = {}
	for i=1,self.number_of_chunks_x do
		self.chunks[i] = {}
		for o=1,self.number_of_chunks_y do
			self.chunks[i][o] = {}
			-- self.chunks[i][o].active = true
			for t=1, self.chunk_size do
				self.chunks[i][o][t] = {}
				for d=1,self.chunk_size do
					self.chunks[i][o][t][d] = {}
				end
			end
		end
	end
	for i=1,self.number_of_chunks_x do
		for v=1,self.number_of_chunks_y do
			for t=1, self.chunk_size do
				for d=1,self.chunk_size do
					local x = ((self.chunk_size*i)-self.chunk_size)+t
					local y = ((self.chunk_size*v)-self.chunk_size)+d
					self.chunks[i][v][t][d].x = (self.block_size*x)-self.block_size
					self.chunks[i][v][t][d].y = (self.block_size*y)-self.block_size
					self.chunks[i][v][t][d].type = self.map[x][y].type
				end
			end
		end
	end
end


function M:update(dt)
	if self.loaded then
		for i=1,self.number_of_chunks_x do
			for v=1,self.number_of_chunks_y do
				local sx = ((self.chunk_size*i)-self.chunk_size)*self.block_size
				local ex = (self.chunk_size*i)*self.block_size
				local sy = ((self.chunk_size*v)-self.chunk_size)*self.block_size
				local ey = (self.chunk_size*v)*self.block_size
				if self.pointer.x > sx and self.pointer.x < ex and self.pointer.y > sy and self.pointer.y < ey then
					self:set_active(i,v)
				end
			end
		end
	end
end

function M:draw()
	if self.loaded then
		for i=1,self.number_of_chunks_x do
			for v=1,self.number_of_chunks_y do
				if self.chunks[i][v].active then
					for t=1, self.chunk_size do
						for d=1,self.chunk_size do
							if self.chunks[i][v][t][d].type == 1 then
								local b = {x = self.chunks[i][v][t][d].x, y =self.chunks[i][v][t][d].y, w =self.block_size,h=self.block_size}
								drawBox(b,0,0,255)
							elseif self.chunks[i][v][t][d].type == 0 then
								local b = {x = self.chunks[i][v][t][d].x, y =self.chunks[i][v][t][d].y, w =self.block_size,h=self.block_size}
								drawBox(b,255,255,255)
							elseif self.chunks[i][v][t][d].type == 2 then
								local b = {x = self.chunks[i][v][t][d].x, y =self.chunks[i][v][t][d].y, w =self.block_size,h=self.block_size}
								drawBox(b,110,110,110)
							elseif self.chunks[i][v][t][d].type == 3 then
								local b = {x = self.chunks[i][v][t][d].x, y =self.chunks[i][v][t][d].y, w =self.block_size,h=self.block_size}
								drawBox(b,0,255,0)
							end
						end
					end
				end
			end
		end
	end
end


function M:smooth(number)
	for i = 1 , number do
		for x=1,self.map_width do
			for y=1,self.map_height do
				local neighbours = self:gs2(x,y)
				if neighbours >= 7 and self.map[x][y] == 0 then
					self.map[x][y].type = 1
				else
					if neighbours > 5 then
						self.map[x][y].type = 1
					elseif neighbours < 4 then
						self.map[x][y].type = 0
					end
				end
			end
		end
	end
end


function M:gs2(x,y)
	local wallcount = 0
	for j = x - 1, x + 1 do
		for d = y - 1, y + 1 do
			if d >= 1 and d <= self.map_height then
				if j ~= x or d ~= y then
					if j == 0 then
						local c = self.map_width
						wallcount = wallcount + self.map[c][d].type
					elseif j > self.map_width then
						local c =  1
						wallcount = wallcount + self.map[c][d].type
					else
						wallcount = wallcount + self.map[j][d].type
					end
				end
			end
		end
	end
	return wallcount
end


function M:test(i, seed,amp,freq)
	local t 
	t = love.math.noise((i * (1/amp) * freq),seed)+
	love.math.noise((i * (1/(amp/2)) * (freq/2)),seed)+
	love.math.noise((i * (1/(amp/4)) * (freq/4)),seed)+
	love.math.noise((i * (1/(amp/8)) * (freq/8)),seed)
	return t 
end


function M:rand()
	for x=1, self.map_width do
		for y=1,self.map_height do
			self.map[x][y].type = 1
		end
	end
	for x=1, self.map_width do
		local h = math.floor(self:test(x/10, self.seed, 100, 60))
		for y=1,h+5 do
			self.map[x][y].type = 0
		end
		for y=h+10, self.map_height do
			local r = love.math.random(1,100)
			if r > self.threshold then
				self.map[x][y].type = 1
			elseif r < self.threshold then
				self.map[x][y].type = 0
			end
		end
	end
end

return M
